"""
domain2idna - The tool to convert a domain or a file with a list
of domain to the famous IDNA format.

This submodule provides everything related to the CLI.

Author:
    Nissar Chababy, @funilrys, contactTATAfunilrysTODTODcom

Contributors:
    Let's contribute to domains2idna!!

Project link:
    https://github.com/PyFunceble/domain2idna

Project documentation:
    http://domain2idna.readthedocs.io

License:
::

    MIT License

    Copyright (c) 2018-2019 Nissar Chababy
    Copyright (c) 2019 PyFunceble

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
"""

import argparse

from colorama import Fore, Style
from colorama import init as initiate_colorama

from . import VERSION
from .converter import Converter
from .helpers import File


def subjects(subject_to_convert, output=None, encoding="utf-8"):
    """
    This function convert the given domain to IDNA format.


    :param str domain_to_convert:
        The domain to convert.
    :param str output:
        The output of the conversion. If not set, we output to stdout.
     :param str encoding:
        The encoding to provide.

    :raise ValueError:
        If the given :code:`domain_to_convert` is empty.
    """

    if subject_to_convert:
        if not isinstance(subject_to_convert, list):
            subject_to_convert = [subject_to_convert]

        converted = Converter(
            subject_to_convert, original_encoding=encoding
        ).get_converted()

        if output:
            File(output).write("\n".join(converted))
        else:
            print("\n".join(converted))
    else:
        raise ValueError("Could not understand <subject_to_convert>.")


def file(file_to_convert, output=None, encoding="utf-8"):
    """
    This function read a file and convert each line of the file to IDNA.

    :param str file_to_convert:
        The file to convert
    :param str output:
        The output of the conversion. If not set, we output to stdout.
    :param str encoding:
        The encoding to provide.
    """

    if file_to_convert:
        converted = []

        try:
            to_convert = File(file_to_convert).read(encoding=encoding).split("\n")
        except (UnicodeEncodeError, UnicodeDecodeError):  # pragma: no cover
            to_convert = File(file_to_convert).read(encoding="ISO-8859-1").split("\n")

        converted = Converter(to_convert).get_converted()

        if output:
            File(output).write("\n".join(converted))
        else:
            print("\n".join(converted))


def tool():  # pragma: no cover
    """
    Provides the CLI.
    """

    if __name__ == "domain2idna.cli":
        initiate_colorama(autoreset=True)

        parser = argparse.ArgumentParser(
            description="domain2idna - The tool to convert a domain or a file with \
            a list of domain to the famous IDNA format.",
            epilog="Crafted with %s by %s"
            % (
                Fore.RED + "♥" + Fore.RESET,
                Style.BRIGHT
                + Fore.CYAN
                + "Nissar Chababy (Funilrys)"
                + Style.RESET_ALL,
            ),
        )

        parser.add_argument("-d", "--domain", type=str, help=argparse.SUPPRESS)

        parser.add_argument(
            "-s",
            "--subject",
            type=str,
            nargs="+",
            help="Sets the subjects to convert.",
            default=[]
        )

        parser.add_argument(
            "-e",
            "--encoding",
            type=str,
            help="Sets the encoding to use.",
            default="utf-8",
        )

        parser.add_argument(
            "-f",
            "--file",
            type=str,
            help="Sets the file to read to get the domain(s) to convert.",
        )

        parser.add_argument(
            "-o",
            "--output",
            type=str,
            help="Sets the file where we write the converted domain(s).",
        )

        parser.add_argument(
            "-v", "--version", action="version", version="%(prog)s " + VERSION
        )

        args = parser.parse_args()

        subject = args.subject

        if args.domain:
            subject.append(args.domain)

        if subject:
            subjects(subject, args.output, encoding=args.encoding)
        elif args.file:
            file(args.file, args.output, encoding=args.encoding)



#!/usr/bin/env python3

"""
domain2idna - The tool to convert a domain or a file with a list
of domain to the famous IDNA format.

Tests the converter itself.

Author:
    Nissar Chababy, @funilrys, contactTATAfunilrysTODTODcom

Contributors:
    Let's contribute to domains2idna!!

Project link:
    https://github.com/PyFunceble/domain2idna

Project documentation:
    http://domain2idna.readthedocs.io

License:
    MIT License

    Copyright (c) 2018-2019 Nissar Chababy
    Copyright (c) 2019 PyFunceble

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
"""

from unittest import TestCase
from unittest import main as launch_tests

from domain2idna.converter import Converter


class TestConverter(TestCase):
    """
    Tests Converter.
    """

    def test_to_idna_single(self):
        """
        Runs and tests Converter
        """

        domain_to_test = "ṁỵetherwallet.com"

        expected = "xn--etherwallet-tv8eq7f.com"
        actual = Converter(domain_to_test).get_converted()

        self.assertEqual(expected, actual)

    def test_to_idna_multiple(self):
        """
        Runs and tests Converter.
        """

        domains_to_test = [
            "bịllogram.com",
            "bittréẋ.com",
            "cryptopiạ.com",
            "coinbȧse.com",
            "cṙyptopia.com",
            "0.0.0.0 ṁỵetherwallet.com",
        ]

        expected = [
            "xn--bllogram-g80d.com",
            "xn--bittr-fsa6124c.com",
            "xn--cryptopi-ux0d.com",
            "xn--coinbse-30c.com",
            "xn--cyptopia-4e0d.com",
            "0.0.0.0 xn--etherwallet-tv8eq7f.com",
        ]
        actual = Converter(domains_to_test).get_converted()

        self.assertEqual(expected, actual)

    def test_to_idna_multiple_urls(self):
        """
        Runs and tests Converter.
        """

        domains_to_test = [
            "http://bịllogram.com",
            "https://bittréẋ.com/path;parameters?query#fragment",
            "ftp://cryptopiạ.com",
            "git://coinbȧse.com",
        ]

        expected = [
            "http://xn--bllogram-g80d.com",
            "https://xn--bittr-fsa6124c.com/path;parameters?query#fragment",
            "ftp://xn--cryptopi-ux0d.com",
            "git://xn--coinbse-30c.com",
        ]
        actual = Converter(domains_to_test).get_converted()

        self.assertEqual(expected, actual)

    def test_commented_line(self):
        """
        Tests that the comments are returned normally.
        """

        comments = [
            "# Hello, World!",
            "# This is another commented line",
            "cryptopiạ.com  # This is a comment at the end of a line.",
            "# This is a commented line with bittréẋ.com",
            "# cryptopiạ.com",
        ]

        expected = [
            "# Hello, World!",
            "# This is another commented line",
            "xn--cryptopi-ux0d.com # This is a comment at the end of a line.",
            "# This is a commented line with bittréẋ.com",
            "# cryptopiạ.com",
        ]

        actual = Converter(comments).get_converted()

        self.assertEqual(expected, actual)

    def test_hosts_file_format(self):
        """
        Tests that the hosts file format is always respected.
        """

        given = [
            "0.0.0.0 bịllogram.com",
            "127.0.0.1 bittréẋ.com",
            "0.0.0.0 cryptopiạ.com",
            "127.0.0.1 coinbȧse.com",
            "0.0.0.0 cṙyptopia.com",
        ]

        expected = [
            "0.0.0.0 xn--bllogram-g80d.com",
            "127.0.0.1 xn--bittr-fsa6124c.com",
            "0.0.0.0 xn--cryptopi-ux0d.com",
            "127.0.0.1 xn--coinbse-30c.com",
            "0.0.0.0 xn--cyptopia-4e0d.com",
        ]

        actual = Converter(given).get_converted()

        self.assertEqual(expected, actual)

        given = "0.0.0.0 bịllogram.com"

        expected = "0.0.0.0 xn--bllogram-g80d.com"

        actual = Converter(given).get_converted()

        self.assertEqual(expected, actual)


if __name__ == "__main__":
    launch_tests()



"""
domain2idna - The tool to convert a domain or a file with a list
of domain to the famous IDNA format.

Provides the base to all other sublogic(s).

Author:
    Nissar Chababy, @funilrys, contactTATAfunilrysTODTODcom

Contributors:
    Let's contribute to domains2idna!!

Project link:
    https://github.com/PyFunceble/domain2idna

Project documentation:
    http://domain2idna.readthedocs.io

License:
    MIT License

    Copyright (c) 2018-2019 Nissar Chababy
    Copyright (c) 2019 PyFunceble

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
"""

from urllib.parse import urlparse


class Converter:
    """
    Provides a base for every core logic we add.

    :param subject: The subject to convert.
    :type subject: str, list

    :param str original_encoding:
        The encoding to provide as output.
    """

    to_ignore = [
        "0.0.0.0",
        "localhost",
        "127.0.0.1",
        "localdomain",
        "local",
        "broadcasthost",
        "allhosts",
        "allnodes",
        "allrouters",
        "localnet",
        "loopback",
        "mcastprefix",
    ]

    def __init__(self, subject, original_encoding="utf-8"):
        self.subject = subject
        self.encoding = original_encoding

    def convert_to_idna(self, subject, original_encoding="utf-8"):
        """
        Converts the given subject to IDNA.

        :param str subject: The subject to convert.
        :rtype: str
        """

        if subject in self.to_ignore:
            return subject

        if "/" not in subject:
            try:
                return subject.encode("idna").decode(original_encoding)
            except UnicodeError:  # pragma: no cover
                return subject

        parsed_url = urlparse(subject)
        result = f"{parsed_url.scheme}://{self.convert_to_idna(parsed_url.netloc)}{parsed_url.path}"

        if parsed_url.params:
            result += f";{parsed_url.params}"
        if parsed_url.query:
            result += f"?{parsed_url.query}"
        if parsed_url.fragment:
            result += f"#{parsed_url.fragment}"

        return result

    def __get_converted(self, subject):
        """
        Process the actual conversion.

        :param str subject: The subject to convert.

        :rtype: str
        """

        if (
            not subject
            or not subject.strip()
            or subject in self.to_ignore
            or subject.startswith("#")
        ):
            return subject

        if "#" in subject and "/" not in subject:
            comment = " " + subject[subject.find("#") :]
            subject = subject[: subject.find("#")].strip()
        else:
            comment = ""

        return (
            " ".join(
                [
                    self.convert_to_idna(x, original_encoding=self.encoding)
                    for x in subject.split()
                ]
            )
            + comment
        ).strip()

    def get_converted(self):
        """
        Provides the converted data.
        """

        if isinstance(self.subject, list):
            return [self.__get_converted(x) for x in self.subject]
        return self.__get_converted(self.subject)
